package query

import (
	"context"
	"encoding/json"
	"fmt"
	"github.com/odysseus/service-registry/common"
	"github.com/redis/go-redis/v9"
)

type NodeManagerInfo struct {
	Timestamp int64  `redis:"timestamp" json:"timestamp"`
	Endpoint  string `redis:"endpoint" json:"endpoint"`
}

func (g NodeManagerInfo) TimeStamp() int64 {
	return g.Timestamp
}

type nodeManagerQuery struct {
	rdb     *redis.Client
	service common.ServiceType
}

func getNodeManagerQuery(rdb *redis.Client) ServiceQuery {
	return nodeManagerQuery{
		rdb:     rdb,
		service: common.SERVICE_NODE_MANAGER,
	}
}

func (g nodeManagerQuery) ModuleName() string {
	return g.service.String()
}

func (g nodeManagerQuery) List() ([]string, error) {
	all, err := getAllNm(g.rdb, g.service)
	if err != nil {
		return nil, err
	}
	var res []string
	for _, v := range all {
		d, _ := json.Marshal(v)
		res = append(res, string(d))
	}
	return res, nil
}

func (g nodeManagerQuery) ServiceInfo(serviceid string) (string, error) {
	info, err := getOneNm(g.rdb, g.service, serviceid)
	if err != nil {
		return "", err
	}
	d, err := json.Marshal(info)
	if err != nil {
		return "", err
	}

	return string(d), nil
}

func getAllNm(rdb *redis.Client, serviceType common.ServiceType) ([]NodeManagerInfo, error) {
	keys, err := rdb.Keys(context.Background(), common.GetServiceKeyPrefix(serviceType)+"*").Result()
	if err != nil {
		return nil, err
	}
	var nmInfos []NodeManagerInfo
	for _, key := range keys {
		res := rdb.HGetAll(context.Background(), key)
		var info NodeManagerInfo
		if err := res.Scan(&info); err != nil {
			continue
		}
		nmInfos = append(nmInfos, info)
	}
	return nmInfos, nil
}

func getOneNm(rdb *redis.Client, serviceType common.ServiceType, endpoint string) (NodeManagerInfo, error) {
	k := fmt.Sprintf("%s%s", common.GetServiceKeyPrefix(serviceType), endpoint)
	res := rdb.HGetAll(context.Background(), k)
	var info NodeManagerInfo
	if err := res.Scan(&info); err != nil {
		return NodeManagerInfo{}, err
	}
	return info, nil
}
